--[[
	Copyright (C) Udorn (Blackhand)
--]]

petbm = petbm or {}
   
petbm.PetTeamView = petbm.PetBattleMaster:NewModule("PetTeamView", "AceEvent-3.0", "AceHook-3.0")
local L = petbm.Locale.GetInstance()

local log = petbm.Debug:new("PetTeamView")

local MAX_ACTIVE_PETS = 3
local MAX_TEAM_FRAMES = 6

local NUM_ICONS_PER_ROW = 5
local NUM_ICON_ROWS = 4
local NUM_ICONS_SHOWN = NUM_ICONS_PER_ROW * NUM_ICON_ROWS
local ICON_ROW_HEIGHT = 36

local BACKDROP_COLOR = {31/255, 28/255, 38/255, 0}
local BACKDROP_BORDER_COLOR = {1, 1, 1, 1}

local BACKDROP = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
	tile = true, tileSize = 16, edgeSize = 16, 
	insets = { left = 4, right = 4, top = 4, bottom = 4 }
}

local function _UpdatePortrait(self, portrait)
	local petId, ability1ID, ability2ID, ability3ID, locked = C_PetJournal.GetPetLoadOutInfo(portrait:GetID())
	if (not petId or petId <= 0) then
		portrait:Hide()
		portrait.healthBar:Hide()
	else
		local speciesID, customName, level, xp, maxXp, displayID, isFavorite, name, icon, petType, creatureId
		if (self.build >= 16257) then
			-- I love Blizzard for this totally incompatible changes, could be made more professional...
			speciesID, customName, level, xp, maxXp, displayID, isFavorite, name, icon, petType, creatureId = C_PetJournal.GetPetInfoByPetID(petId)
		else
			speciesID, customName, level, xp, maxXp, displayID, name, icon, petType, creatureId = C_PetJournal.GetPetInfoByPetID(petId)
		end
		if (petType and petType > 0) then
			local health, maxHealth, attack, speed, quality = C_PetJournal.GetPetStats(petId)
			portrait.petTypeIcon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType])
			portrait.icon:SetTexture(icon)
			portrait.level:SetText(level)
			portrait:Show()
			portrait.healthBar:Hide()
			local color = ITEM_QUALITY_COLORS[1]
			if (quality) then
				color = ITEM_QUALITY_COLORS[quality-1]
			end
			portrait.iconBorder:SetVertexColor(color.r, color.g, color.b, 1)
			
			if (health <= 0) then
				portrait.isDead:Show()
			else
				portrait.isDead:Hide()
				if (health < maxHealth - 1) then
					portrait.healthBar:Update(health, maxHealth)
					portrait.healthBar:Show()
				end
			end
		else
			portrait:Hide()
			portrait.healthBar:Hide()
		end
	end
end

local function _SaveTeam(self)

	if (self.selectTeamProcess) then
		return
	end
	
	log:Debug("_SaveTeam [%s]", self.currentTeam.name)
	if (not self.currentTeam.abilities) then
		self.currentTeam.abilities = {}
	end
	for i=1,MAX_ACTIVE_PETS do
		if (not self.currentTeam.abilities[i]) then
			self.currentTeam.abilities[i] = {}
		end
		local abilities = self.currentTeam.abilities[i]
		self.currentTeam.pets[i], abilities[1], abilities[2], abilities[3] = C_PetJournal.GetPetLoadOutInfo(i)
		log:Debug("_SaveTeam petId[%s]=[%s] abilities[%s %s %s]", i, self.currentTeam.pets[i], abilities[1], abilities[2], abilities[3])
	end
end

-- The team selection is called asynchronously, otherwise
-- the abilities doesn't get updated correctly sometimes.
-- After each step the process is delayed by one frame.
local function _SelectTeamProcess(self)

	log:Debug("SelectTeamProcess")
	
	for petIndex=1,MAX_ACTIVE_PETS do
		local curAbilities = self.abilities
		local curPetId
		curPetId, curAbilities[1], curAbilities[2], curAbilities[3] = C_PetJournal.GetPetLoadOutInfo(petIndex)
		local petId = self.currentTeam.pets[petIndex]
		log:Debug("select [%s]=[%s] ([%s])", petIndex, petId, curPetId)
		if (petId) then
			if (petId ~= curPetId) then
				C_PetJournal.SetPetLoadOutInfo(petIndex, petId)
				return
			else
				local abilities = self.currentTeam.abilities[petIndex]
				if (abilities) then
					for abilityIndex=1,3 do
						local abilityId = abilities[abilityIndex] 
						log:Debug("cmp ability[%s][%s]=[%s] ([%s])", petIndex, abilityIndex, abilityId, curAbilities[abilityIndex])
						if (abilityId and curAbilities[abilityIndex] ~= abilityId) then
							log:Debug("set ability[%s][%s]=[%s]", petIndex, abilityIndex, abilityId)
							C_PetJournal.SetAbility(petIndex, abilityIndex, abilityId)
							return
						end
					end
				end
			end
		end
	end

	if (PetJournal_UpdatePetLoadOut) then
		PetJournal_UpdatePetLoadOut()
	end
	
--	self.selectDelay = time() + 1
	self.selectTeamProcess = nil
end

local function _SelectTeam(self, id)
	log:Debug("SelectTeam [%s]", id)
	self.db.profile.currentTeam = id
	self.currentTeam = self.db.profile.teams[id]
	self.teamName:SetText(self.currentTeam.name)
	self.selectTeamProcess = true
end

local function _OnTeamSelected(frame)
	_SelectTeam(frame.obj, frame:GetID())
end

local function _GetTeamSelectTooltip(frame)
	local self = frame.obj
	local team = self.db.profile.teams[frame:GetID()]
	if (team) then
		return L["Selects the team \"%s\""]:format(team.name)
	end
	return ""
end

local function _GetTeamEditTooltip(frame)
	local self = frame.obj
	if (self.currentTeam) then
		return L["Edits the name and the icon of team \"%s\""]:format(self.currentTeam.name)
	end
	return ""
end

local function _UpdateTeamFrames(self)
	for i=1,MAX_TEAM_FRAMES do
		local teamFrame = self.teamFrames[i]
		local team = self.db.profile.teams[i]
		teamFrame.icon:SetTexture(team.icon)
	end
end

local function _UpdateTeamFrame(frame)
	local self = frame.obj
	local hidden = self.isSelecting or (self.selectDelay and time() < self.selectDelay)
	if (hidden) then
		frame:Disable()
		frame.icon:SetDesaturated(1)
	else
		frame:Enable()
		frame.icon:SetDesaturated(nil)
		if (self.currentTeam) then
			self.teamName:SetText(self.currentTeam.name)
		end
		_UpdateTeamFrames(self)
	end
	if (frame:GetID() == self.db.profile.currentTeam) then
		frame.iconBorder:Show()
	else
		frame.iconBorder:Hide()
	end
end

local function _OnShowPopup(popup)
	local self = popup.obj
	if (not self.currentTeam) then
		popup:Hide()
		return
	end
	self.selectedTexture = nil
	popup.editBox:SetText(self.currentTeam.name)
end

local function _OnSavePopup(ok)
	local self = ok:GetParent().obj
	local popup = self.popup
	if (not self.currentTeam) then
		popup:Hide()
		return
	end
	self.currentTeam.name = popup.editBox:GetText() or ""
	if (self.selectedTexture) then
		log:Debug("set icon")
		self.currentTeam.icon = "INTERFACE\\ICONS\\"..self.selectedTexture
	end
	popup:Hide()
end

local function _UpdatePopup(scrollFrame)
	local popup = scrollFrame:GetParent()
	local self = popup.obj
	local numIcons = #self.icons
	local offset = FauxScrollFrame_GetOffset(popup.scrollFrame)
	local index = (offset * NUM_ICONS_PER_ROW) + 1
	
	log:Debug("offset [%s]", offset)
	
	local texture
	for row=1, NUM_ICON_ROWS do
		local iconLine = popup["iconLine"..row]
		for col=1, NUM_ICONS_PER_ROW do
			local icon = iconLine["icon"..col]
			if (index > numIcons) then
				icon:Hide()
			else
				local texture = self.icons[index]
				if (texture) then
					icon.icon:SetTexture("INTERFACE\\ICONS\\"..texture)
					icon:Show()
					if (texture == self.selectedTexture) then
						icon:SetChecked(1)
					else
						icon:SetChecked(nil)
					end
				else
					icon:Hide()
				end
				index = index + 1
			end
		end
	end
	FauxScrollFrame_Update(popup.scrollFrame, ceil(numIcons / NUM_ICONS_PER_ROW) , NUM_ICON_ROWS, ICON_ROW_HEIGHT)
		
--	for i=1, NUM_ICONS_SHOWN do
--		local macroPopupIcon = _G["MacroPopupButton"..i.."Icon"]
--		local macroPopupButton = _G["MacroPopupButton"..i]
--		index = (macroPopupOffset * NUM_ICONS_PER_ROW) + i;
--		texture = GetSpellorMacroIconInfo(index);
--		if ( index <= numMacroIcons and texture ) then
--			macroPopupIcon:SetTexture("INTERFACE\\ICONS\\"..texture);
--			macroPopupButton:Show();
--		else
--			macroPopupIcon:SetTexture("");
--			macroPopupButton:Hide();
--		end
--		if ( MacroPopupFrame.selectedIcon and (index == MacroPopupFrame.selectedIcon) ) then
--			macroPopupButton:SetChecked(1);
--		elseif ( MacroPopupFrame.selectedIconTexture ==  texture ) then
--			macroPopupButton:SetChecked(1);
--		else
--			macroPopupButton:SetChecked(nil);
--		end
--	end
--	
--	-- Scrollbar stuff
--	FauxScrollFrame_Update(MacroPopupScrollFrame, ceil(numMacroIcons / NUM_ICONS_PER_ROW) , NUM_ICON_ROWS, MACRO_ICON_ROW_HEIGHT );
end

local function _TogglePopup(frame)
	local self = frame.obj
	if (self.popup:IsVisible()) then
		self.popup:Hide()
	else
		self.popup:Show()
		_UpdatePopup(self.popup.scrollFrame)
	end
end

local function _InitPopup(self)
	local popup = self.popup
	popup.obj = self
	popup.nameLabel:SetText(L["Team name"])
	popup:SetScript("OnShow", _OnShowPopup)
	popup.ok:SetScript("OnClick", _OnSavePopup)
	popup.scrollFrame:SetScript("OnVerticalScroll", function(frame, offset)
		FauxScrollFrame_OnVerticalScroll(frame, offset, ICON_ROW_HEIGHT, _UpdatePopup)
	end)
end
 
local function _InitPortraitFrame(self, parent, id)
	local frame = CreateFrame("Frame", "petbmTeamMember"..id, parent, "petbmTeamMemberTemplate") 
	frame:SetID(id)
	petbm.WidgetUtil.AddTooltip(frame, L["Pet portrait"], L["Enter pet journal to switch between pets"])
	
	local healthBar = petbm.HealthBar.New(parent, frame:GetWidth() - 2, 8)
	healthBar.frame:SetPoint("TOPLEFT", frame, "TOPLEFT", 4, 0)
	frame.healthBar = healthBar
	
	return frame
end

local function _InitTeamFrame(self, parent, id)
	local frame = CreateFrame("Button", "petbmTeam"..id, parent, "petbmTeamTemplate")
	self.teamFrames[id] = frame
	frame.obj = self
	frame:SetID(id)
	frame:SetScript("OnClick", _OnTeamSelected)
	frame:SetScript("OnUpdate", _UpdateTeamFrame)
	
	petbm.WidgetUtil.AddTooltip(frame, L["Team selection"], _GetTeamSelectTooltip)
	
	return frame
end

local function _InitFrame(self)
	local frame = CreateFrame("Frame", "petbmTeamView", UIParent)--, "SecureFrameTemplate")
	frame.obj = self
	frame:SetWidth(270)
	frame:SetHeight(140)
	frame:SetPoint("TOP", UIParent, "TOP", 0, -30)
	frame:SetScript("OnUpdate", function(frame)
		if (frame.obj.selectTeamProcess) then
			_SelectTeamProcess(frame.obj)
		end 
		frame.obj:Update() 
	end)
	frame:SetMovable(true)
	frame:Hide()
	self.frame = frame
	petbm.MovingView.RegisterFrame(frame)
	
	frame:SetBackdrop(BACKDROP)
	frame:SetBackdropColor(unpack(BACKDROP_COLOR))
	
	local background = frame:CreateTexture(nil, "BACKGROUND")
	self.background = background
	background:SetPoint("TOPLEFT", frame, "TOPLEFT", 4, -4)
	background:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -4, 4)
	--background:SetTexture("Interface\\PetBattles\\MountJournal-BG")
	background:SetTexture("Interface\\PetBattles\\PetJournal")
	background:SetTexCoord(0.00195313, 0.79882813, 0.59277344, 0.75976563)
	--background:SetTexCoord(0.00195313, 0.79101563, 0.48730469, 0.59082031)

	-- pet menu
	local petJournal = CreateFrame("Button", nil, frame, "MainMenuBarMicroButton")
	petJournal:SetPoint("TOPLEFT", frame, "TOPLEFT", 15, -24)
	LoadMicroButtonTextures(petJournal, "Mounts")
	petJournal.tooltipText = MicroButtonTooltipText(MOUNTS_AND_PETS, "TOGGLEPETJOURNAL")
	petJournal.newbieText = NEWBIE_TOOLTIP_MOUNTS_AND_PETS
	petJournal:SetScript("OnClick", function() TogglePetJournal(2) end)	
	
	-- team portraits
	local portrait1 = _InitPortraitFrame(self, frame, 1)
	portrait1:SetPoint("LEFT", petJournal, "RIGHT", 10, 0)
	self.portrait1 = portrait1
	
	local portrait2 = _InitPortraitFrame(self, frame, 2)
	portrait2:SetPoint("LEFT", portrait1, "RIGHT", 20, 0)
	self.portrait2 = portrait2

	local portrait3 = _InitPortraitFrame(self, frame, 3)
	portrait3:SetPoint("LEFT", portrait2, "RIGHT", 20, 0)
	self.portrait3 = portrait3

	-- change team name and icon
	local change = CreateFrame("Button", nil, frame)
	change.obj = self
	change:SetWidth(16)
	change:SetHeight(16)
	local texture = change:CreateTexture()
	texture:SetTexture("Interface\\Addons\\PetBattleMaster\\resources\\Edit-Up")
	texture:SetAllPoints(change)
	texture:SetTexCoord(0, 19/32, 0, 18/32)
	change:SetNormalTexture(texture)
	local texture = change:CreateTexture()
	texture:SetTexture("Interface\\Addons\\PetBattleMaster\\resources\\Edit-Down")
	texture:SetAllPoints(change)
	texture:SetTexCoord(0, 19/32, 0, 18/32)
	change:SetPushedTexture(texture)
	change:SetPoint("TOPLEFT", frame, "TOPLEFT", 12, -12)
	change:SetScript("OnClick", _TogglePopup)
	petbm.WidgetUtil.AddTooltip(change, L["Team name/icon edit"], _GetTeamEditTooltip)

	-- team name
	local teamName = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	teamName:SetJustifyH("LEFT")
	teamName:SetWidth(200)
	teamName:SetPoint("LEFT", change, "RIGHT", 10, 0)
--	teamName:SetPoint("TOPLEFT", frame, "TOPLEFT", 12, -12)
	teamName:SetText("Team 1")
	self.teamName = teamName
	
	-- close button
	local close = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
	close:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, 0)
	close:SetScript("OnClick", function(frame) frame:GetParent():Hide() end)
	
	-- team buttons
	for i=1,MAX_TEAM_FRAMES do
		local teamButton = _InitTeamFrame(self, frame, i)
		if (i == 1) then
			teamButton:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 12, 7)
		else
			teamButton:SetPoint("LEFT", self.teamButtons[i - 1], "RIGHT", 2, 0)
		end
		self.teamButtons[i] = teamButton
	end
	
	-- heal button
	-- TODO the parent has also to be secure, but that's not enough...
--	local heal = CreateFrame("Button", nil, frame, "PetbmHealPetButtonTemplate")
--	heal:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -15, -30)
--	self.heal = heal
		
end

local function ProfileUpdate(self)
	_UpdateTeamFrames(self)
end

local function SetPetLoadOutInfo(self)
	_SaveTeam(self)
end

local function SetAbility(self)
	_SaveTeam(self)
end

local function PET_BATTLE_CLOSE(self)
	if (petbm.PetBattleMaster.db.profile.autoOpenTeamView) then
		self.frame:Show()
	end
end

local function PET_BATTLE_OPENING_START(self)
	self.frame:Hide()
end

local function Update(self)
--	if (self.heal and PetJournalHealPetButton_OnLoad) then
--		PetJournalHealPetButton_OnLoad(self.heal)
--		self.heal = nil
--	end
	
--	if (not self.hooked and PetJournalDragButton_OnDragStart) then
--		log:Debug("HOOK")
--		self:SecureHook("PetJournalDragButton_OnDragStart", function() log:Debug("DRAG") end)
--		self.hooked = true
--	end
	self.background:SetAlpha(petbm.PetBattleMaster.db.profile.teamViewAlpha)
	self.frame:SetBackdropBorderColor(unpack(petbm.PetBattleMaster.db.profile.teamViewBorderColor))
	_UpdatePortrait(self, self.portrait1)
	_UpdatePortrait(self, self.portrait2)
	_UpdatePortrait(self, self.portrait3)
end

local function IconButton_OnClick(frame, button, down)
	local self = petbm.PetTeamView
	local popup = self.popup
	log:Debug("icon click frame: [%s]", frame)
	local row = button:GetParent():GetID() - 1
	local col = button:GetID()
	local index = (FauxScrollFrame_GetOffset(popup.scrollFrame) * NUM_ICONS_PER_ROW) + (row * NUM_ICONS_PER_ROW) + col
	log:Debug("index [%s]", index)
	if (index <= #self.icons) then
		self.selectedTexture = self.icons[index]
		log:Debug("texture [%s]", self.selectedTexture)
	end 
--	MacroPopupButton_SelectTexture(self:GetID() + ( * NUM_ICONS_PER_ROW));
	_UpdatePopup(popup.scrollFrame)
end

local function OnInitialize(self)
	self.version, self.build = GetBuildInfo()
	self.build = tonumber(self.build) -- why the hell is that a string?!
	self.teamButtons = {}
	self.teamFrames = {}
	self.abilities = {}
	self.db = petbm.PetBattleMaster.db:RegisterNamespace("PetTeamView", {
		profile = {
			currentTeam = 1,
    		teams = {
    			{
    				name = L["Team %s"]:format(1),
    				icon = "Interface\\Icons\\ability_deathknight_summongargoyle",
    				pets = {}
    			},
    			{
    				name = L["Team %s"]:format(2),
    				icon = "Interface\\Icons\\Ability_Druid_FerociousBite",
    				pets = {}
    			},
    			{
    				name = L["Team %s"]:format(3),
    				icon = "Interface\\Icons\\Ability_EyeOfTheOwl",
    				pets = {}
    			},
    			{
    				name = L["Team %s"]:format(4),
    				icon = "Interface\\Icons\\Ability_Hunter_Pet_Devilsaur",
    				pets = {}
    			},
    			{
    				name = L["Team %s"]:format(5),
    				icon = "Interface\\Icons\\achievment_Boss_ultraxion",
    				pets = {}
    			},
    			{
    				name = L["Team %s"]:format(6),
    				icon = "Interface\\Icons\\INV_Pet_Cats_OrangeTabbyCat",
    				pets = {}
    			}
    		}
    	}
    })
    self.currentTeam = self.db.profile.teams[self.db.profile.currentTeam]
	_InitFrame(self)
end

local function OnEnable(self)
	self:RegisterEvent("PET_BATTLE_OPENING_START")
	self:RegisterEvent("PET_BATTLE_CLOSE")
	self:SecureHook(C_PetJournal, "SetPetLoadOutInfo")
	self:SecureHook(C_PetJournal, "SetAbility")
	
	self.popup = _G["petbmNameIconEditor"]
	self.icons = {}
	GetMacroIcons(self.icons)
	GetMacroItemIcons(self.icons)
	
	_InitPopup(self)
	self:ProfileUpdate()
end

local function ToggleView(self)
	if (self.frame:IsVisible()) then
		self.frame:Hide()
	else
		self.frame:Show()
		_SelectTeam(self, self.db.profile.currentTeam)
	end	
end

petbm.PetTeamView.OnInitialize = OnInitialize
petbm.PetTeamView.OnEnable = OnEnable
petbm.PetTeamView.ProfileUpdate = ProfileUpdate
petbm.PetTeamView.Update = Update
petbm.PetTeamView.ToggleView = ToggleView
petbm.PetTeamView.PET_BATTLE_CLOSE = PET_BATTLE_CLOSE
petbm.PetTeamView.PET_BATTLE_OPENING_START = PET_BATTLE_OPENING_START
petbm.PetTeamView.SetPetLoadOutInfo = SetPetLoadOutInfo
petbm.PetTeamView.SetAbility = SetAbility
petbm.PetTeamView.IconButton_OnClick = IconButton_OnClick